{.pl72}

(* Letzte Bearbeitung : 22.12.1992   15:22 *)

(*

   P A S C A L  -  B I B L I O T H E K   fr die Makrobefehle

   des SPB-Moduls M-SIO-8 von SORCUS COMPUTER GmbH

   Version 1A (Z80-I/O-Befehle)    (nderungen vorbehalten)

*)

(*
  Globale Variable
                  *)

CONST
     CCTS : byte = 0;    (* CTS-Pegel *)
     CRTS : byte = 0;    (* RTS-Pegel *)

(* Initialisierung eines Kanals auf dem SPB-Modul M-SIO-8 *)

procedure ml4_initsio8(modul,kanal: byte;
                       rbaud,tbaud: integer;
                       laenge,stopb,parity,cts,rts: byte);
const
     cmd : byte = 0;

var
   err1,zwivar : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      zwivar := $35;                                           (* Transmitter enable, Receiver enable,
                                                                  RESET Error, Receiver Interrupt enabled,
                                                                  Transmitter Interrupt disabled,
                                                                  Kein Transmit Break, Betriebsart normal *)

      if ml4_err = 0 then ml4_outport(ml4_err,cmd+3,zwivar);   (* KCR = zwivar *)
      zwivar := 0;
      case stopb of                                            (* Stop-Bits *)
        1 : zwivar := $40;                                     (* 1 Stop-Bit *)
        2 : zwivar := $C0;                                     (* 2 Stop-Bits *)
        5 : zwivar := $80;                                     (* 1,5 Stop-Bits *)
        end;
      case parity of                                           (* Paritt *)
        1 : zwivar := zwivar + $10;                      (* ungerade Paritt *)
        2 : zwivar := zwivar + $30;                      (* gerade Paritt *)
        0 : zwivar := zwivar + $00;                      (* keine Paritt *)
        end;
       case laenge of                                          (* Lnge *)
        5: zwivar := zwivar + $00;                             (* 5 Bit *)
        6: zwivar := zwivar + $04;                             (* 6 Bit *)
        7: zwivar := zwivar + $08;                             (* 7 Bit *)
        8: zwivar := zwivar + $0C;                             (* 8 Bit *)
        end;
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+2,zwivar);   (* MR1 = zwivar *)
      zwivar := 0;
      case rbaud of                                            (* Empfangs-Baudrate *)
        50    : zwivar := $00;
        75    : zwivar := $01;
        110   : zwivar := $02;
        134   : zwivar := $03;
        150   : zwivar := $04;
        300   : zwivar := $05;
        600   : zwivar := $06;
        1200  : zwivar := $07;
        1800  : zwivar := $08;
        2000  : zwivar := $09;
        2400  : zwivar := $0A;
        3600  : zwivar := $0B;
        4800  : zwivar := $0C;
        7200  : zwivar := $0D;
        9600  : zwivar := $0E;
        19200 : zwivar := $0F;
        end;
      case tbaud of                                            (* Sende-Baudrate *)
        50    : zwivar := zwivar + $00;
        75    : zwivar := zwivar + $10;
        110   : zwivar := zwivar + $20;
        134   : zwivar := zwivar + $30;
        150   : zwivar := zwivar + $40;
        300   : zwivar := zwivar + $50;
        600   : zwivar := zwivar + $60;
        1200  : zwivar := zwivar + $70;
        1800  : zwivar := zwivar + $80;
        2000  : zwivar := zwivar + $90;
        2400  : zwivar := zwivar + $A0;
        3600  : zwivar := zwivar + $B0;
        4800  : zwivar := zwivar + $C0;
        7200  : zwivar := zwivar + $D0;
        9600  : zwivar := zwivar + $E0;
        19200 : zwivar := zwivar + $F0;
        end;
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+2,zwivar);   (* MR2 = zwivar *)
      CCTS := cts;
      CRTS := rts;
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Lies Interrupt-Status-Register auf dem Modul M-SIO-8 *)

procedure ml4_sio8_getint(modul: byte;var inter : boolean;var kanal : byte);
const
     cmd : byte = 0;
     stat : byte = 0;

var
   err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_inport(ml4_err,stat,cmd+4);      (* ISR lesen *)
      err1 := ml4_err;
      if (stat and $80) = $80 then                             (* Interrupt-Request ? *)
        begin
          kanal := (stat and $0F) shr 1;                       (* Kanal ermitteln *)
          inter := true;                                       (* Interrupt-Anforderung liegt vor *)
        end
      else
        begin
          kanal := 0;                                          (* Default-Zuweisung *)
          inter := false;                                      (* Keine Interrupt-Anforderung *)
        end;
    end
  else                                                         (* Falscher Steckplatz *)
    begin
      err1 := 20;
    end;
  ml4_err := err1;
end;


(* Lies Status eines Kanals des SPB-Moduls M-SIO-8 *)

function ml4_sio8_gets(modul,kanal : byte) : byte;
const
     cmd : byte = 0;
     data : byte = 0;

var
   err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+1);      (* data = KSR *)
      err1 := ml4_err;
      ml4_sio8_gets := data and $BB;
    end
  else                                                         (* Falscher Steckplatz *)
    begin
      err1 := 20;
      ml4_sio8_gets := 0;
    end;
  ml4_err := err1;
end;

(* Senden eines Zeichens ber einen Kanal des SPB-Moduls M-SIO-8 *)

procedure ml4_sio8_tmt(modul,kanal: byte;zeichen : char);
const
     cmd : byte = 0;

var
   data,err1 : byte;
   zustand : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then
        ml4_outport(ml4_err,cmd,ord(zeichen));                 (* KDR = zeichen *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Clear Error-Status des SPB-Moduls M-SIO-8 *)

procedure ml4_sio8_clr(modul,kanal : byte);
const
     cmd : byte = 0;

var
   data,err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+3);      (* KCR lesen *)
      data := data or $10;                                     (* Error-Bit = 1 setzen *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+3,data);     (* KCR = data *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Setze Transmit Break des SPB-Moduls M-SIO-8 *)

procedure ml4_sio8_brk(modul,kanal : byte);
const
     cmd : byte = 0;

var
   data,err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+3);      (* KCR lesen *)
      data := data or $08;                                     (* Transmit Break = 1 setzen *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+3,data);     (* KCR = data *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Hebe Transmit Break des SPB-Moduls M-SIO-8 auf *)

procedure ml4_sio8_ubrk(modul,kanal : byte);
const
     cmd : byte = 0;

var
   data,err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+3);      (* KCR lesen *)
      data := data and $F7;                                    (* Transmit Break = 0 setzen *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+3,data);     (* KCR = data *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Setzen der RTS-Leitung eines Kanals des SPB-Moduls M-SIO-8 = 0 *)

procedure ml4_sio8_0rts(modul,kanal : byte);
const
     cmd : byte = 0;

var
   err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      cmd := cmd + $10 + (kanal*2) + (1 - CRTS);
      if ml4_err = 0 then ml4_outport(ml4_err,cmd,0);          (* RTS = 0 *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;


(* Setzen der RTS-Leitung eines Kanals des SPB-Moduls M-SIO-8 = 1 *)

procedure ml4_sio8_1rts(modul,kanal : byte);
const
     cmd : byte = 0;

var
   err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      cmd := cmd + $10 + (kanal*2) + CRTS;
      if ml4_err = 0 then ml4_outport(ml4_err,cmd,0);          (* RTS = 1 *)
      err1 := ml4_err;
    end
  else                                                         (* Falscher Steckplatz *)
    err1 := 20;
  ml4_err := err1;
end;

(* Empfangen eines Zeichens ber einen Kanal des SPB-Moduls M-SIO-8 *)

function ml4_sio8_rcvb(modul,kanal,status : byte) : char;
const
     cmd : byte = 0;
     data : byte = 0;

var
   loop: integer;
   err1 : byte;

begin
  if modul in [1..4] then
    begin
      cmd := ((modul - 1) * $20) + $80;                        (* Befehlscode ermitteln *)
      if ml4_err = 0 then ml4_outport(ml4_err,cmd+8,kanal*2);  (* KAR = kanal*2 *)
      if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+1);      (* KSR lesen *)
      if (data and $2) = $2 then
        begin                                                  (* Character Received ? *)
          ml4_sio8_0rts(modul,kanal);
          if ml4_err = 0 then ml4_inport(ml4_err,data,cmd);    (* data = RDR *)
        end
      else
        begin
          loop := 0;
          ml4_sio8_1rts(modul,kanal);
          repeat
            if ml4_err = 0 then ml4_inport(ml4_err,data,cmd+1);    (* KSR lesen *)
            loop := succ(loop);
          until ((data and $2) = $2) or (loop = 1000);                                (* Character Received ? *)
          if (data and $2) = $2 then
            if ml4_err = 0 then ml4_inport(ml4_err,data,cmd)       (* data = RDR *)
          else
            data := 0;
        end;
      if ml4_err = 0 then ml4_inport(ml4_err,status,cmd+1);    (* KSR lesen *)
      if loop = 1000 then
        status := status and $0B
      else
        status := status and $BB;
      err1 := ml4_err;
      ml4_sio8_rcvb := chr(data);
    end
  else                                                         (* Falscher Steckplatz *)
    begin
      err1 := 20;
      ml4_sio8_rcvb := chr(0);
    end;
  ml4_err := err1;
end;


